@@ -1,19 +1,19 @@
 """
 Helper classes for parsers.
 """
-from __future__ import absolute_import, unicode_literals
 
+import contextlib
 import datetime
 import decimal
 import json  # noqa
 import uuid
 
 from django.db.models.query import QuerySet
-from django.utils import six, timezone
-from django.utils.encoding import force_text
+from django.utils import timezone
+from django.utils.encoding import force_str
 from django.utils.functional import Promise
 
-from rest_framework.compat import coreapi, total_seconds
+from rest_framework.compat import coreapi
 
 
 class JSONEncoder(json.JSONEncoder):
@@ -23,9 +23,9 @@ class JSONEncoder(json.JSONEncoder):
     """
     def default(self, obj):
         # For Date Time string spec, see ECMA 262
-        # http://ecma-international.org/ecma-262/5.1/#sec-15.9.1.15
+        # https://ecma-international.org/ecma-262/5.1/#sec-15.9.1.15
         if isinstance(obj, Promise):
-            return force_text(obj)
+            return force_str(obj)
         elif isinstance(obj, datetime.datetime):
             representation = obj.isoformat()
             if representation.endswith('+00:00'):
@@ -39,17 +39,17 @@ class JSONEncoder(json.JSONEncoder):
             representation = obj.isoformat()
             return representation
         elif isinstance(obj, datetime.timedelta):
-            return six.text_type(total_seconds(obj))
+            return str(obj.total_seconds())
         elif isinstance(obj, decimal.Decimal):
             # Serializers will coerce decimals to strings by default.
             return float(obj)
         elif isinstance(obj, uuid.UUID):
-            return six.text_type(obj)
+            return str(obj)
         elif isinstance(obj, QuerySet):
             return tuple(obj)
-        elif isinstance(obj, six.binary_type):
+        elif isinstance(obj, bytes):
             # Best-effort for binary blobs. See #4187.
-            return obj.decode('utf-8')
+            return obj.decode()
         elif hasattr(obj, 'tolist'):
             # Numpy arrays and array scalars.
             return obj.tolist()
@@ -59,10 +59,20 @@ class JSONEncoder(json.JSONEncoder):
                 'You should be using a schema renderer instead for this view.'
             )
         elif hasattr(obj, '__getitem__'):
-            try:
-                return dict(obj)
-            except:
-                pass
+            cls = (list if isinstance(obj, (list, tuple)) else dict)
+            with contextlib.suppress(Exception):
+                return cls(obj)
         elif hasattr(obj, '__iter__'):
             return tuple(item for item in obj)
-        return super(JSONEncoder, self).default(obj)
+        return super().default(obj)
+
+
+class CustomScalar:
+    """
+    CustomScalar that knows how to encode timedelta that renderer
+    can understand.
+    """
+    @classmethod
+    def represent_timedelta(cls, dumper, data):
+        value = str(data.total_seconds())
+        return dumper.represent_scalar('tag:yaml.org,2002:str', value)
